/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 *
 * Contributor(s):
 *
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.resource.pool;

import com.sun.enterprise.connectors.ConnectorConnectionPool;
import com.sun.enterprise.resource.ResourceSpec;

import javax.resource.ResourceException;
import javax.resource.spi.ManagedConnection;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;
import com.sun.logging.LogDomains;
import com.sun.appserv.connectors.internal.api.PoolingException;

/**
 * Abstract Pool manager for unimplemented features. Throws UnsupportedOperationException when invoked.
 */
public abstract class AbstractPoolManager implements PoolManager {
    /**
     * Obtain a transactional resource such as JDBC connection
     *
     * @param spec  Specification for the resource
     * @param alloc Allocator for the resource
     * @param info  Client security for this request
     * @return An object that represents a connection to the resource
     * @throws PoolingException Thrown if some error occurs while
     *                          obtaining the resource
     */

    private static Logger _logger = null;

    static {
        _logger = LogDomains.getLogger(AbstractPoolManager.class, LogDomains.RSR_LOGGER);
    }

    public void emptyResourcePool(ResourceSpec spec) {
        throw new UnsupportedOperationException();
    }

    public void disableMonitoring(String poolName) {
        throw new UnsupportedOperationException();
    }

    public void setMonitoringEnabledHigh(String poolName) {
        throw new UnsupportedOperationException();
    }

    public void setMonitoringEnabledLow(String poolName) {
        throw new UnsupportedOperationException();
    }//get the pooltable

   //register the MonitoringLevelListeners
    public void initializeMonitoring() {
        throw new UnsupportedOperationException();
    }

    public void killAllPools() {
        throw new UnsupportedOperationException();
    }

    public void killFreeConnectionsInPools() {
        throw new UnsupportedOperationException();
    }

    public void setSelfManaged(String poolName, boolean flag) {
        throw new UnsupportedOperationException();
    }

    public void lazyEnlist(ManagedConnection mc) throws ResourceException {
        throw new UnsupportedOperationException();
    }

    public ConcurrentHashMap getMonitoredPoolTable() {
        throw new UnsupportedOperationException();
    }
}
